﻿using ProjectAirFighter.CollectionGenericObject;
using ProjectAirFighter.CollectionGenericObjects;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectAirFighter.CollectionGenericObjects;
/// <summary>
/// Параметризованный набор объектов
/// </summary>
/// <typeparam name="T">Параметр: ограничение - ссылочный тип</typeparam>
public class MassiveGenericObjects<T> : ICollectionGenericObjects<T>
    where T : class
{
    /// <summary>
    /// Массив объектов, которые хроним
    /// </summary>
    private T[] _collection;

    public int Count => _collection.Length;

    public int SetMaxCount { set { if (value > 0) { _collection = new T?[value]; } } }

    /// <summary>
    /// Конструктор
    /// </summary>
    public MassiveGenericObjects()
    {
        _collection = Array.Empty<T?>();
    }

    public T? Get(int position)
    {
        if (position >= 0 && position < Count)
        {
            return _collection[position];
        }
        return null;
    }

    public int Insert(T obj)
    {
        for (int i = 0; i < Count; i++)
        {
            if (_collection[i] == null)
            {
                _collection[i] = obj;
                return i;
            }
        }
        return -1;
    }

    public int Insert(T obj, int position)
    {
        if (position < 0 || position >= Count)
            return -1;

        if (_collection[position] == null)
        {
            _collection[position] = obj;
            return position;
        }

        int temp = position + 1;
        while (temp < Count)
        {
            if (_collection[temp] == null)
            {
                _collection[temp] = obj;
                return temp;
            }
            temp++;
        }

        temp = position - 1;
        while (temp > 0)
        {
            if (_collection[temp] == null)
            {
                _collection[temp] = obj;
                return temp;
            }
            temp--;
        }

        return -1;
    }

    public T? Remove(int position)
    {
        if (position < 0 || position >= Count)
            return null;

        if (_collection[position] == null)
        {
            return null;
        }

        T? temp = _collection[position];
        _collection[position] = null;
        return temp;
    }
}
