﻿using ProjectAirFighter.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectAirFighter.Drawning;

public class DrawningWarPlane
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityWarPlane? EntityWarPlane { get; protected set; }

    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;

    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;

    /// <summary>
    /// Левая координата прорисовки самолета
    /// </summary>
    protected int? _startPosX;

    /// <summary>
    /// Верхняя координата прорисовки самолета
    /// </summary>
    protected int? _startPosY;

    /// <summary>
    /// Ширина прорисовки самолета
    /// </summary>
    private readonly int _drawningWarPlaneWidth = 120;

    /// <summary>
    /// Высота прорисовки самолета
    /// </summary>
    private readonly int _drawningWarPlaneHeight = 140;

    /// <summary>
    /// Координаты X объекта
    /// </summary>
    public int? GetPosX => _startPosX;

    /// <summary>
    /// Координаты Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;

    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight => _drawningWarPlaneHeight;

    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWigdth => _drawningWarPlaneWidth;
    private DrawningWarPlane()
    {
        _pictureHeight = null;
        _pictureWidth = null;
        _startPosX = null;
        _startPosY = null;
    }
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    public DrawningWarPlane(int speed, double weight, Color bodyColor) : this()
    {
        EntityWarPlane = new EntityWarPlane(speed, weight, bodyColor);

    }
    /// <summary>
    /// Конструктор для наслдеников
    /// </summary>
    /// <param name="drawningWarPlaneHeight">Высота прорисовки самолета</param>
    /// <param name="drawningWarPlaneWidth">Ширина прорисовки самолета</param>
    public DrawningWarPlane(int drawningWarPlaneWidth, int drawningWarPlaneHeight) : this()
    {
        _drawningWarPlaneWidth = drawningWarPlaneWidth;
        _drawningWarPlaneHeight = drawningWarPlaneHeight;

    }


    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns></returns>
    public bool SetPictureSize(int width, int height)
    {
        if (width > _drawningWarPlaneWidth && height > _drawningWarPlaneHeight)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_startPosX != null && _startPosY != null)
            {
                if (_startPosX.Value < 0)
                {
                    _startPosX = 0;
                }
                if (_startPosY.Value < 0)
                {
                    _startPosY = 0;
                }
                if (_startPosX.Value + _drawningWarPlaneWidth > _pictureWidth)
                {
                    _startPosX = _pictureWidth - _drawningWarPlaneWidth;
                }
                if (_startPosY.Value + _drawningWarPlaneHeight > _pictureHeight)
                {
                    _startPosY = _pictureHeight - _drawningWarPlaneHeight;
                }
            }

            return true;
        }
        return false;

    }

    /// <summary>
    /// Установка позиции 
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        else
        {
            _startPosX = x;
            _startPosY = y;
            if (_startPosX.Value < 0)
            {
                _startPosX = 0;
            }
            if (_startPosY.Value < 0)
            {
                _startPosY = 0;
            }
            if (_startPosX + _drawningWarPlaneWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth - _drawningWarPlaneWidth;
            }
            if (_startPosY + _drawningWarPlaneHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight - _drawningWarPlaneHeight;
            }
        }

    }




    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещение выполнено, false - перемещение невозможно </returns>
    public virtual bool MoveTransport(DirectionType direction)
    {
        if (EntityWarPlane == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }
        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityWarPlane.Step > 0)
                {
                    _startPosX -= (int)EntityWarPlane.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityWarPlane.Step > 0)
                {
                    _startPosY -= (int)EntityWarPlane.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + EntityWarPlane.Step + _drawningWarPlaneWidth < _pictureHeight)
                {
                    _startPosY += (int)EntityWarPlane.Step;
                }
                return true;
            //вправо
            case DirectionType.Right:
                if (_startPosX.Value + EntityWarPlane.Step + _drawningWarPlaneWidth < _pictureWidth)
                {
                    _startPosX += (int)EntityWarPlane.Step;
                }
                return true;
            default:
                return false;
        }
    }

    public virtual void DrawTransport(Graphics g)
    {
        if (EntityWarPlane == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        Pen pen = new(Color.Black, 3);
        Pen pen_rocket = new(Color.Black);

        //корпус
        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 60, 100, 20);
        Point[] body = { new Point(_startPosX.Value + 100, _startPosY.Value + 80),
            new Point(_startPosX.Value + 120, _startPosY.Value + 70), new Point(_startPosX.Value + 100, _startPosY.Value + 60) };
        g.DrawPolygon(pen, body);
        //нижнее крыло
        Point[] wingUpper = { new Point(_startPosX.Value + 80, _startPosY.Value + 80),
            new Point(_startPosX.Value + 80, _startPosY.Value + 140),  new Point(_startPosX.Value + 70, _startPosY.Value + 140),
            new Point(_startPosX.Value + 60, _startPosY.Value + 80) };
        g.DrawPolygon(pen, wingUpper);
        //верхнее крыло
        Point[] wingLower =  { new Point(_startPosX.Value + 80, _startPosY.Value + 60), new Point(_startPosX.Value + 80, _startPosY.Value),
            new Point(_startPosX.Value + 70, _startPosY.Value), new Point(_startPosX.Value + 60, _startPosY.Value + 60) };
        g.DrawPolygon(pen, wingLower);
        //задние крылья
        //верхнее
        Point[] rearWingUpper = { new Point(_startPosX.Value, _startPosY.Value + 60), new Point(_startPosX.Value, _startPosY.Value + 30),
            new Point(_startPosX.Value + 25, _startPosY.Value + 50), new Point(_startPosX.Value + 25, _startPosY.Value + 60)};
        g.DrawPolygon(pen, rearWingUpper);
        //нижнее
        Point[] rearWingLower = { new Point(_startPosX.Value, _startPosY.Value + 80),
            new Point(_startPosX.Value, _startPosY.Value + 110),new Point(_startPosX.Value + 25, _startPosY.Value + 90),
            new Point(_startPosX.Value + 25, _startPosY.Value + 80) };
        g.DrawPolygon(pen, rearWingLower);

        //залив носа
        Brush brBlack = new SolidBrush(Color.Black);
        g.FillPolygon(brBlack, body);

        //залив корпуса
        Brush br = new SolidBrush(EntityWarPlane.BodyColor);
        g.FillRectangle(br, _startPosX.Value, _startPosY.Value + 60, 100, 20);

        g.FillPolygon(br, wingLower);
        g.FillPolygon(br, wingUpper);
        g.FillPolygon(br, rearWingUpper);
        g.FillPolygon(br, rearWingLower);


    }
}
